{===============================================================================}
{               B&L Service                                                     }
{               homepage : www.b-und-l-service.de                               }
{               email    : info@www.b-und-l-service                             }
{               J. Beushausen                                                   }
{===============================================================================}
{ 09.09.2015    demo program, client for serial interface                       }
{===============================================================================}

unit SerialClient;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IniFiles, AdPacket, OoMisc, AdPort, CPort, ExtCtrls;

const
  paOk       = 0;
  paError    = 1;

  cmdRX      = 1;
  cmdTX      = 2;
  cmdSave    = 3;
  cmdVersion = 4;
  cmdReset   = 5;

  cmdNoCommand = 255;

  CR = #13;
  LF = #10;

  strVersion = 'serial server V 1.0';

type
  TfrmSerialClient = class(TForm)
    meRX: TMemo;
    ComPort1: TComPort;
    paTop: TPanel;
    Label2: TLabel;
    cbComPortNr: TComboBox;
    cbBaudRate: TComboBox;
    edTX: TEdit;
    btOpenPort: TButton;
    btSend: TButton;
    btClosePort: TButton;
    procedure FormCreate(Sender: TObject);
    procedure cbComPortNrChange(Sender: TObject);
    procedure cbBaudRateChange(Sender: TObject);
    procedure btOpenPortClick(Sender: TObject);
    procedure btClosePortClick(Sender: TObject);
    procedure ComPort1RxChar(Sender: TObject; Count: Integer);
    procedure btSendClick(Sender: TObject);
  private
    strRX : String;
    bPort, bBaud : boolean;

    procedure SetComPortNr(ComPortNr : integer);
    procedure SetBaudRate(BaudRate: integer);
    function  OpenComPort: integer;
    function  CloseComPort: integer;
    procedure Init;
    procedure SendStr(str: string);

    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  frmSerialClient: TfrmSerialClient;

implementation

{$R *.dfm}

{------------------------------------------------------------------------------}

procedure TfrmSerialClient.Init;
begin
  strRX := '';
  meRX.Lines.Clear;
  cbComPortNr.Text := 'select port';
  cbBaudRate.Text  := 'select baud rate';
  btOpenPort.Enabled := false;
  btSend.Enabled     := false;
  bPort := false;
  bBaud := false;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.FormCreate(Sender: TObject);
begin
  Init;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.SetComPortNr(ComPortNr : integer);
begin
  ComPort1.Port := 'COM' + IntToStr(ComPortNr);
  bPort := true;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.cbComPortNrChange(Sender: TObject);
var
  ComPortNr : integer;
begin
  ComPortNr := cbComPortNr.ItemIndex + 1;
  SetComPortNr(ComPortNr);
  btOpenPort.Enabled := bBaud and bPort;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.SetBaudRate(BaudRate : integer);
var
  br : TBaudRate;
begin
  case BaudRate of
    9600   : br := br9600;
    38400  : br := br38400;
    115200 : br := br115200;
  else
    br := br9600;
  end;

  ComPort1.BaudRate := br;
  bBaud := true;

  btOpenPort.Enabled := bBaud and bPort;
  btOpenPort.Enabled := bBaud and bPort;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.cbBaudRateChange(Sender: TObject);
var
  BaudRate : integer;
begin
  BaudRate := strToInt(cbBaudRate.Items[cbBaudRate.ItemIndex]);
  SetBaudRate(BaudRate);
end;
{------------------------------------------------------------------------------}

function TfrmSerialClient.OpenComPort : integer;
begin
  ComPort1.Open;
  if ComPort1.Connected then
    Result := 0
  else
    Result := 1;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.btOpenPortClick(Sender: TObject);
var
  iErr : integer;
begin
  iErr := OpenComPort;
  if iErr <> 0 then
    MessageDlg('error open port', mtError, [mbCancel], 0)
  else
  begin
    meRX.Lines.Add('com port open');
    btSend.Enabled := true;
  end;
end;
{------------------------------------------------------------------------------}

function TfrmSerialClient.CloseComPort : integer;
begin
  ComPort1.Close;
  if  not ComPort1.Connected then
    Result := 0
  else
    Result := 1;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.btClosePortClick(Sender: TObject);
var
  iErr : integer;
begin
  iErr := CloseComPort;
  Init;

  if iErr <> 0 then
    MessageDlg('error close port', mtError, [mbCancel], 0)
  else
    meRX.Lines.Add('com port close');
end;
{------------------------------------------------------------------------------}

procedure Delay(Milliseconds: cardinal);
var
  Tick  : cardinal;
  Event : THandle;
begin
  Event := CreateEvent(nil, False, False, nil);
  try
    Tick := GetTickCount + Milliseconds;
    while (Milliseconds > 0) and (MsgWaitForMultipleObjects(1, Event, False, Milliseconds, QS_ALLINPUT) <> WAIT_TIMEOUT) do
    begin
      Application.ProcessMessages;
      if Application.Terminated then
          Exit;
      Milliseconds := Tick - GetTickcount;
    end;
  finally
    CloseHandle(Event);
  end;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.ComPort1RxChar(Sender: TObject; Count: Integer);
var
  i : Word;
  c : Char;
  str : string;
begin
  ComPort1.ReadStr(str, count);
  strRX := strRX + str;

  for i := 1 to count do begin
    c := str[i];
    if c = LF then begin
      meRX.Lines.Add(strRX);
    end;
  end;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.SendStr(str : string);
var
  strTX : string;
begin
   strTX :=  str + CR + LF;
   ComPort1.WriteStr(strTX);
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialClient.btSendClick(Sender: TObject);
begin
  SendStr(edTX.Text);
  edTX.Text := '';
end;

end.
