{===============================================================================}
{               B&L Service                                                     }
{               homepage : www.b-und-l-service.de                               }
{               email    : info@www.b-und-l-service                             }
{                          j.beushausen@gmx.de                                  }
{               J. Beushausen                                                   }
{===============================================================================}
{ 09.09.2015    simulation, server for serial interface  V 1.0                  }
{===============================================================================}

unit SerialServerSimulation;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, IniFiles, AdPacket, OoMisc, AdPort, CPort, ExtCtrls;

const
  paOk       = 0;
  paError    = 1;

  cmdRX      = 1;
  cmdTX      = 2;
  cmdSave    = 3;
  cmdVersion = 4;
  cmdReset   = 5;

  cmdNoCommand = 255;

  CR = #13;
  LF = #10;

  strVersion = 'serial server simulation, V 1.0';

type
  TfrmSerialServer = class(TForm)
    meRX: TMemo;
    ComPort1: TComPort;
    paTop: TPanel;
    cbSimulation: TCheckBox;
    Label2: TLabel;
    cbComPortNr: TComboBox;
    cbBaudRate: TComboBox;
    btOpenPort: TButton;
    btClosePort: TButton;
    procedure FormCreate(Sender: TObject);
    procedure cbComPortNrChange(Sender: TObject);
    procedure cbBaudRateChange(Sender: TObject);
    procedure btOpenPortClick(Sender: TObject);
    procedure btClosePortClick(Sender: TObject);
    procedure ComPort1RxChar(Sender: TObject; Count: Integer);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    strRX : String;
    bPort, bBaud : boolean;
    b_TempSimulation : byte;
    _iStep : integer;
    ParameterFld : array[0..25 ] of single;

    procedure SetComPortNr(ComPortNr : integer);
    procedure SetBaudRate(BaudRate: integer);
    function  OpenComPort: integer;
    function  CloseComPort: integer;
    procedure Init;

    procedure StoreParameter(index: byte; value: single);
    procedure SendParameter(index: byte);
    procedure SendValue(aValue: single);
    procedure ClrParameterFld;
    procedure SendVersion;
    procedure doReset;
    procedure doSave;
    function  GetTempRaw() : byte;
    function  CalculateTemp(bTemp: Byte; sOffset, sSlope: Single): Single;
    function  ParseRXStr(strRX: string; var bCommand: byte; var index: byte; var sValue: single): integer;
    function  CreateIni(var aIniFile: TIniFile): boolean;
    procedure SaveParameters(values: array of single);
    function  GetParameters(var values: array of single): boolean;
    function  SimulateTemperatureMeasurment : single;
    procedure DefaultParameters(var values: array of single);
    function  ComPortIsOpen: boolean;
    procedure SendError;
    procedure SendOk;

    procedure Delay(Milliseconds: longint);
    procedure AppIdle(Sender: TObject; var Done: Boolean);
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  frmSerialServer: TfrmSerialServer;

implementation

{$R *.dfm}

{------------------------------------------------------------------------------}

procedure TfrmSerialServer.Init;
begin
  strRX := '';
  meRX.Lines.Clear;
  cbComPortNr.Text := 'select port';
  cbBaudRate.Text  := 'select baud rate';
  btOpenPort.Enabled := false;
  bPort := false;
  bBaud := false;
  b_TempSimulation := 0;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.ClrParameterFld;
var
  b : byte;
begin
  for b := 0 to length(ParameterFld) - 1 do
    ParameterFld[b] := 0;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.FormCreate(Sender: TObject);
begin
  Init;
  ClrParameterFld;
  if not GetParameters(ParameterFld) then begin
    DefaultParameters(ParameterFld);
    SaveParameters(ParameterFld);
  end;

  Application.OnIdle := AppIdle;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.SetComPortNr(ComPortNr : integer);
begin
  ComPort1.Port := 'COM' + IntToStr(ComPortNr);
  bPort := true;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.cbComPortNrChange(Sender: TObject);
var
  ComPortNr : integer;
begin
  ComPortNr := cbComPortNr.ItemIndex + 1;
  SetComPortNr(ComPortNr);
  btOpenPort.Enabled := bBaud and bPort;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.SetBaudRate(BaudRate : integer);
var
  br : TBaudRate;
begin
  case BaudRate of
    9600   : br := br9600;
    38400  : br := br38400;
    115200 : br := br115200;
  else
    br := br9600;
  end;

  ComPort1.BaudRate := br;
  bBaud := true;

  btOpenPort.Enabled := bBaud and bPort;
  btOpenPort.Enabled := bBaud and bPort;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.cbBaudRateChange(Sender: TObject);
var
  BaudRate : integer;
begin
  BaudRate := strToInt(cbBaudRate.Items[cbBaudRate.ItemIndex]);
  SetBaudRate(BaudRate);
end;
{------------------------------------------------------------------------------}

function TfrmSerialServer.OpenComPort : integer;
begin
  ComPort1.Open;
  if ComPort1.Connected then
    Result := 0
  else
    Result := 1;
end;
{------------------------------------------------------------------------------}

function TfrmSerialServer.ComPortIsOpen : boolean;
begin
  Result := ComPort1.Connected;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.btOpenPortClick(Sender: TObject);
var
  iErr : integer;
begin
  iErr := OpenComPort;
  if iErr <> 0 then
    MessageDlg('error open port', mtError, [mbCancel], 0)
  else
    meRX.Lines.Add('com port open');
end;
{------------------------------------------------------------------------------}

function TfrmSerialServer.CloseComPort : integer;
begin
  ComPort1.Close;
  if  not ComPort1.Connected then
    Result := 0
  else
    Result := 1;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.btClosePortClick(Sender: TObject);
var
  iErr : integer;
begin
  iErr := CloseComPort;
  Init;
  Application.ProcessMessages;

  if iErr <> 0 then
    MessageDlg('error close port', mtError, [mbCancel], 0)
  else
    meRX.Lines.Add('com port close');
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.Delay(Milliseconds: longint);
var
  Tick  : longint;
  Event : THandle;
begin
  Event := CreateEvent(nil, False, False, nil);
  try
    Tick := GetTickCount + Milliseconds;
    while (Milliseconds > 0) and (MsgWaitForMultipleObjects(1, Event, False, Milliseconds, QS_ALLINPUT) <> WAIT_TIMEOUT) do
    begin
      Application.ProcessMessages;
      if Application.Terminated then
          Exit;
      Milliseconds := Tick - GetTickcount;
    end;
  finally
    CloseHandle(Event);
  end;
end;
{------------------------------------------------------------------------------}

function isValidValueChar(aChar : char) : byte;
var
  _asc : Byte;
  _valid : Byte;
begin
  _valid := 1;
  _asc := ord(aChar);

  if (_asc < ord('0')) or (_asc > ord('9')) then begin
    if (_asc <> ord('.')) and (_asc <> ord(',')) and (_asc <> ord('+')) and (_asc <> ord('-')) and (_asc <> ord(';')) and (_asc <> ord('e')) and (_asc <> ord('E')) then
       _valid := 0
  end;

  isValidValueChar := _valid;
end;
{------------------------------------------------------------------------------}

function isValidIndexChar(aChar : char) : byte;
var
  _asc : Byte;
  _valid : Byte;
begin
  _valid := 0;
  _asc := ord(aChar);

  if ((_asc >= ord('a')) and (_asc <= ord('z'))) or ((_asc >= ord('A')) and (_asc <= ord('Z'))) then
    _valid := 1
  else
  begin
    if (_asc = ord('!')) or (_asc >= ord('?')) or (_asc <= ord('$')) then
      _valid := 1
  end;

  isValidIndexChar := _valid;
end;
{------------------------------------------------------------------------------}

function TfrmSerialServer.ParseRXStr(strRX: string; var bCommand: byte; var index: byte; var sValue: single): integer;
var
 strVal:  String;
 strIndex:  String;
 len, i, _index: integer;
 hasSemicolon : boolean;
 chCmd : char;
 char1 : char;
begin
  {parse string}
  Result := paError;
  hasSemicolon := false;
  strRX := trim(strRX);
  
  len := length(strRX);

  strVal    := '';
  strIndex  := '';
  sValue    := 0;
  bCommand  := cmdNoCommand;

  for i := 1 to len do begin
    char1 := strRX[i];
    if not hasSemicolon then begin
      if isValidIndexChar(char1) = 1 then
        strIndex := strIndex + char1;
      hasSemicolon := char1 = ';'
    end
    else
    begin
      if isValidValueChar(char1) = 1 then begin
        if (char1 = ',') or (char1 = '.') then
          Char1 := decimalSeparator;

        strVal := strVal + char1;
      end;
    end;
  end;

  if length(strIndex) >= 1 then begin
     chCmd := strRX[1];
     _index := ord(chCmd);

     {lower case character}
     if (_index >= 97) and (_index <= 122) then  begin {a .. z}
       index := _index - 96;       // 1 .. 25
       bCommand := cmdTX;
       Result := paOk;
     end;

     {upper case character}
     if (_index >= 65) and (_index <= 90) then begin  {A .. Z}
       index := _index - 64;    // 1 .. 25
       bCommand := cmdRX;
       Result := paOk;
     end;

     {special command character}
     if (_index = 33) or (_index = 36) or (_index = 63) then begin    {!, $ or ?}
       case _index of
           33  : bCommand  := cmdReset;        { ! }
           36  : bCommand  := cmdSave;         { $ }
           63  : bCommand  := cmdVersion;      { ? }
        end;
       Result := paOk;
     end
  end
  else
    Result := paError;

  if Result = paOk then begin
    if strVal <> '' then begin
      try
        sValue := strToFloat(strVal)
      except
        Result := paError;
      end;
    end  
    else
    begin
      if hasSemicolon then   {with semicolon but without value}
        Result := paError;
    end;

    if (bCommand = cmdRX) and (strVal = '') then
        Result := paError;
  end;
end;
{------------------------------------------------------------------------------}

function TfrmSerialServer.CreateIni(var aIniFile: TIniFile): boolean;
var
  AppPath, AppName, IniName, IniFileName : string;
begin
  AppPath     := ExtractFilePath(Application.ExeName);
  AppName     := ExtractFileName(Application.ExeName);
  IniName     := copy(AppName, 1, Pos('.', AppName)) + 'ini';
  IniFileName := AppPath + IniName;

  Result := FileExists(IniFileName);
  try
    aIniFile := TIniFile.Create(IniFileName);
  except
  end;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.SaveParameters(values : array of single);
var
  aIniFile : TIniFile;
  i : integer;
  strPar : string;
begin
  CreateIni(aIniFile);

  for i := 0 to length(values) - 1 do begin
     strPar := 'par ' + chr(i + 64);
     aIniFile.WriteFloat('parameter', strPar, values[i]);
  end;
  aIniFile.UpdateFile;

  aIniFile.Destroy;
end;
{------------------------------------------------------------------------------}

function TfrmSerialServer.GetParameters(var values : array of single) : boolean;
var
  aIniFile : TIniFile;
  i : integer;
  strPar : string;
begin
  if CreateIni(aIniFile) then begin;
    Result := true;
    for i := 0 to length(values) - 1 do begin
       strPar := 'par ' + chr(i + 64);
       values[i] := aIniFile.ReadFloat('parameter', strPar, 0);
    end;
    aIniFile.Destroy;
  end
  else
    Result := false;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.StoreParameter(index : byte; value: single);
begin
  ParameterFld[index] := value;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.DefaultParameters(var values : array of single);
begin
   values[ord('O') - 64] := -20.0;
   values[ord('S') - 64] := 0.549;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.SendParameter(index : byte);
var
  strTX : string;
begin
  strTX := FloatToStr(ParameterFld[index]) + CR + LF;
  ComPort1.WriteStr(strTX);
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.SendOk;
var
  strTX : string;
begin
  strTX := 'ok' + CR + LF;
  ComPort1.WriteStr(strTX);
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.SendError;
var
  strTX : string;
begin
  strTX := 'error' + CR + LF;
  ComPort1.WriteStr(strTX);
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.SendValue(aValue : single);
var
  strTX : string;
begin
  strTX := FloatToStr(aValue) + CR + LF;
  ComPort1.WriteStr(strTX);
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.SendVersion;
var
  strTX : string;
begin
  strTX := strVersion + CR + LF;
  ComPort1.WriteStr(strTX);
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.doReset;
begin
  meRX.Lines.Add('reset');
  ClrParameterFld;
  Defaultparameters(ParameterFld);
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.doSave;
begin
  meRX.Lines.Add('save parameter');
  SaveParameters(ParameterFld);
end;
{------------------------------------------------------------------------------}

function TfrmSerialServer.GetTempRaw():  Byte;
begin
  if (b_TempSimulation >= 251) then
    _iStep := -2;
  if (b_TempSimulation <= 0) then
      _iStep := 4;

  b_TempSimulation := b_TempSimulation + _iStep;

  Result := b_TempSimulation;
end;
{------------------------------------------------------------------------------}

function TfrmSerialServer.CalculateTemp(bTemp : Byte; sOffset, sSlope : Single) :  Single;
var
  _sHelp: Single;
begin
  _sHelp := bTemp*sSlope;
  CalculateTemp := sOffset + _sHelp;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.ComPort1RxChar(Sender: TObject; Count: Integer);
var
  i : Word;
  c : Char;
  index, cmdNumber : byte;
  parseResult : integer;
  value : single;
  str : string;
begin
  ComPort1.ReadStr(str, count);
  strRX := strRX + str;

  Application.ProcessMessages;      {important for OnIdle}

  for i := 1 to Count do begin
    c := str[i];
    if c = LF then begin

      meRX.Lines.Add('received ' + strRX);
      parseResult := ParseRXStr(strRX,  cmdNumber, index, value);

      if ParseResult = paOk then begin
        meRX.Lines.Add('command number ' + intToStr(cmdNumber));

        case cmdNumber of
          cmdRX      : StoreParameter(index, value);
          cmdTX      : SendParameter(index);
          cmdVersion : SendVersion;
          cmdReset   : doReset;
          cmdSave    : doSave;
        else
          strRX := '';
        end;

        case cmdNumber of
          cmdRX      : SendOk;
          cmdReset   : SendOk;
          cmdSave    : SendOk;
        end;
      end
      else
      begin
         meRX.Lines.Add('error ' + strRX);
         SendError;
      end;
      strRX := '';
    end; {of if c = LF}
  end;
end;
{------------------------------------------------------------------------------}

function TfrmSerialServer.SimulateTemperatureMeasurment : single;
var
  b : byte;
  sOffset, sSlope : Single;
begin
  {0 .. 255 ref. to -20 .. 120 grd C}
  b := getTempRaw();

  sOffset := ParameterFld[ord('O') - 64];
  sSlope  := ParameterFld[ord('S')-  64];

  Result := CalculateTemp(b, sOffset, sSlope);
  meRX.Lines.Add('raw  ' + intToStr(b) + ' T ' + format('%5.2f', [Result]) + 'C');
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.AppIdle(Sender: TObject; var Done: Boolean);
var
  T : single;
begin
  if cbSimulation.Checked and ComPortIsOpen then begin
    delay(1000);
    T := SimulateTemperatureMeasurment;
    SendValue(T);
    done := false;
  end;
end;
{------------------------------------------------------------------------------}

procedure TfrmSerialServer.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  try
    CloseComPort;
  except
  end;
end;
{------------------------------------------------------------------------------}

end.
